@@ -22,8 +22,9 @@ from code_puppy.plugins.chatgpt_oauth.config import get_chatgpt_models_path
 from code_puppy.plugins.claude_code_oauth.config import get_claude_models_path
 
 from . import callbacks
+from .claude_cache_client import ClaudeCacheAsyncClient, patch_anthropic_client_messages
 from .config import EXTRA_MODELS_FILE
-from .http_utils import create_async_client
+from .http_utils import create_async_client, get_cert_bundle_path, get_http2
 from .round_robin_model import RoundRobinModel
 
 # Environment variables used in this module:
@@ -220,10 +221,28 @@ class ModelFactory:
                     f"API key is not set for Claude Code endpoint; skipping model '{model_config.get('name')}'."
                 )
                 return None
-            client = create_async_client(headers=headers, verify=verify)
-            anthropic_client = AsyncAnthropic(
-                base_url=url, http_client=client, auth_token=api_key
+
+            # Use a dedicated client wrapper that injects cache_control on /v1/messages
+            if verify is None:
+                verify = get_cert_bundle_path()
+
+            http2_enabled = get_http2()
+
+            client = ClaudeCacheAsyncClient(
+                headers=headers,
+                verify=verify,
+                timeout=180,
+                http2=http2_enabled,
             )
+
+            anthropic_client = AsyncAnthropic(
+                base_url=url,
+                http_client=client,
+                auth_token=api_key,
+            )
+            # Ensure cache_control is injected at the Anthropic SDK layer too
+            # so we don't depend solely on httpx internals.
+            patch_anthropic_client_messages(anthropic_client)
             anthropic_client.api_key = None
             anthropic_client.auth_token = api_key
             provider = AnthropicProvider(anthropic_client=anthropic_client)
