@@ -29,6 +29,7 @@ from code_puppy.tui.components import (
     ChatView,
     CustomTextArea,
     InputArea,
+    RightSidebar,
     Sidebar,
     StatusBar,
 )
@@ -40,6 +41,7 @@ from .screens import (
     HelpScreen,
     MCPInstallWizardScreen,
     ModelPicker,
+    QuitConfirmationScreen,
     SettingsScreen,
     ToolsScreen,
 )
@@ -51,20 +53,28 @@ class CodePuppyTUI(App):
     TITLE = "Code Puppy - AI Code Assistant"
     SUB_TITLE = "TUI Mode"
 
+    # Enable beautiful Nord theme by default
+    # Available themes: "textual-dark", "textual-light", "nord", "gruvbox",
+    # "catppuccin-mocha", "catppuccin-latte", "dracula", "tokyo-night", "monokai", etc.
+    DEFAULT_THEME = "nord"
+
     CSS = """
     Screen {
         layout: horizontal;
+        background: $surface;
     }
 
     #main-area {
         layout: vertical;
         width: 1fr;
         min-width: 40;
+        background: $panel;
     }
 
     #chat-container {
         height: 1fr;
         min-height: 10;
+        background: $surface;
     }
     """
 
@@ -78,6 +88,7 @@ class CodePuppyTUI(App):
         Binding("ctrl+4", "show_tools", "Tools"),
         Binding("ctrl+5", "focus_input", "Focus Prompt"),
         Binding("ctrl+6", "focus_chat", "Focus Response"),
+        Binding("ctrl+7", "toggle_right_sidebar", "Status"),
         Binding("ctrl+t", "open_mcp_wizard", "MCP Install Wizard"),
     ]
 
@@ -126,11 +137,27 @@ class CodePuppyTUI(App):
         self._current_worker = None
         self.initial_command = initial_command
 
+        # Set the theme - you can change this to any Textual built-in theme
+        # Try: "nord", "gruvbox", "dracula", "tokyo-night", "monokai", etc.
+        self.theme = self.DEFAULT_THEME
+
         # Initialize message queue renderer
         self.message_queue = get_global_queue()
         self.message_renderer = TUIRenderer(self.message_queue, self)
         self._renderer_started = False
 
+        # Track session start time
+        from datetime import datetime
+
+        self._session_start_time = datetime.now()
+
+        # Background worker for periodic context updates during agent execution
+        self._context_update_worker = None
+
+        # Track double-click timing for history list
+        self._last_history_click_time = None
+        self._last_history_click_index = None
+
     def compose(self) -> ComposeResult:
         """Create the UI layout."""
         yield StatusBar()
@@ -139,6 +166,7 @@ class CodePuppyTUI(App):
             with Container(id="chat-container"):
                 yield ChatView(id="chat-view")
             yield InputArea()
+        yield RightSidebar()
         yield Footer()
 
     def on_mount(self) -> None:
@@ -201,6 +229,14 @@ class CodePuppyTUI(App):
         if self.initial_command:
             self.call_after_refresh(self.process_initial_command)
 
+        # Initialize right sidebar (hidden by default)
+        try:
+            right_sidebar = self.query_one(RightSidebar)
+            right_sidebar.display = True  # Show by default for sexy UI
+            self._update_right_sidebar()
+        except Exception:
+            pass
+
     def _tighten_text(self, text: str) -> str:
         """Aggressively tighten whitespace: trim lines, collapse multiples, drop extra blanks."""
         try:
@@ -456,6 +492,8 @@ class CodePuppyTUI(App):
                     self._current_worker = None
                     self.agent_busy = False
                     self.stop_agent_progress()
+                    # Stop periodic context updates
+                    self._stop_context_updates()
             except Exception as e:
                 self.add_error_message(f"Failed to cancel processing: {str(e)}")
                 # Only clear state on exception if we haven't already done so
@@ -466,6 +504,8 @@ class CodePuppyTUI(App):
                     self._current_worker = None
                     self.agent_busy = False
                     self.stop_agent_progress()
+                    # Stop periodic context updates
+                    self._stop_context_updates()
 
     async def process_message(self, message: str) -> None:
         """Process a user message asynchronously."""
@@ -474,6 +514,9 @@ class CodePuppyTUI(App):
             self._update_submit_cancel_button(True)
             self.start_agent_progress("Thinking")
 
+            # Start periodic context updates
+            self._start_context_updates()
+
             # Handle commands
             if message.strip().startswith("/"):
                 # Handle special commands directly
@@ -538,6 +581,9 @@ class CodePuppyTUI(App):
                     # Refresh history display to show new interaction
                     self.refresh_history_display()
 
+                    # Update right sidebar with new token counts
+                    self._update_right_sidebar()
+
                 except Exception as eg:
                     # Handle TaskGroup and other exceptions
                     # BaseExceptionGroup is only available in Python 3.11+
@@ -561,6 +607,9 @@ class CodePuppyTUI(App):
             self._update_submit_cancel_button(False)
             self.stop_agent_progress()
 
+            # Stop periodic context updates and do a final update
+            self._stop_context_updates()
+
     # Action methods
     def action_clear_chat(self) -> None:
         """Clear the chat history."""
@@ -570,6 +619,15 @@ class CodePuppyTUI(App):
         agent.clear_message_history()
         self.add_system_message("Chat history cleared")
 
+    def action_quit(self) -> None:
+        """Show quit confirmation dialog before exiting."""
+
+        def handle_quit_confirmation(should_quit: bool) -> None:
+            if should_quit:
+                self.exit()
+
+        self.push_screen(QuitConfirmationScreen(), handle_quit_confirmation)
+
     def action_show_help(self) -> None:
         """Show help information in a modal."""
         self.push_screen(HelpScreen())
@@ -579,7 +637,7 @@ class CodePuppyTUI(App):
         sidebar = self.query_one(Sidebar)
         sidebar.display = not sidebar.display
 
-        # If sidebar is now visible, focus the history list to enable immediate keyboard navigation
+        # If sidebar is now visible, focus the history list to enable keyboard navigation
         if sidebar.display:
             try:
                 # Ensure history tab is active
@@ -593,37 +651,13 @@ class CodePuppyTUI(App):
                 history_list = self.query_one("#history-list", ListView)
                 history_list.focus()
 
-                # If the list has items, get the first item for the modal
+                # If the list has items, set the index to the first item
                 if len(history_list.children) > 0:
                     # Reset sidebar's internal index tracker to 0
                     sidebar.current_history_index = 0
-
                     # Set ListView index to match
                     history_list.index = 0
 
-                    # Get the first item and show the command history modal
-                    first_item = history_list.children[0]
-                    if hasattr(first_item, "command_entry"):
-                        # command_entry = first_item.command_entry
-
-                        # Use call_after_refresh to allow UI to update first
-                        def show_modal():
-                            from .components.command_history_modal import (
-                                CommandHistoryModal,
-                            )
-
-                            # Get all command entries from the history list
-                            command_entries = []
-                            for i, child in enumerate(history_list.children):
-                                if hasattr(child, "command_entry"):
-                                    command_entries.append(child.command_entry)
-
-                            # Push the modal screen
-                            # The modal will get the command entries from the sidebar
-                            self.push_screen(CommandHistoryModal())
-
-                        # Schedule modal to appear after UI refresh
-                        self.call_after_refresh(show_modal)
             except Exception as e:
                 # Log the exception in debug mode but silently fail for end users
                 import logging
@@ -656,6 +690,18 @@ class CodePuppyTUI(App):
         chat_view = self.query_one("#chat-view", ChatView)
         chat_view.focus()
 
+    def action_toggle_right_sidebar(self) -> None:
+        """Toggle right sidebar visibility."""
+        try:
+            right_sidebar = self.query_one(RightSidebar)
+            right_sidebar.display = not right_sidebar.display
+
+            # Update context info when showing
+            if right_sidebar.display:
+                self._update_right_sidebar()
+        except Exception:
+            pass
+
     def action_show_tools(self) -> None:
         """Show the tools modal."""
         self.push_screen(ToolsScreen())
@@ -851,6 +897,88 @@ class CodePuppyTUI(App):
         """Stop showing agent progress indicators."""
         self.set_agent_status("Ready", show_progress=False)
 
+    def _update_right_sidebar(self) -> None:
+        """Update the right sidebar with current session information."""
+        try:
+            right_sidebar = self.query_one(RightSidebar)
+
+            # Get current agent and calculate tokens
+            agent = get_current_agent()
+            message_history = agent.get_message_history()
+
+            total_tokens = sum(
+                agent.estimate_tokens_for_message(msg) for msg in message_history
+            )
+            max_tokens = agent.get_model_context_length()
+
+            # Calculate session duration
+            from datetime import datetime
+
+            duration = datetime.now() - self._session_start_time
+            hours = int(duration.total_seconds() // 3600)
+            minutes = int((duration.total_seconds() % 3600) // 60)
+
+            if hours > 0:
+                duration_str = f"{hours}h {minutes}m"
+            else:
+                duration_str = f"{minutes}m"
+
+            # Update sidebar
+            right_sidebar.update_context(total_tokens, max_tokens)
+            right_sidebar.update_session_info(
+                message_count=len(message_history),
+                duration=duration_str,
+                model=self.current_model,
+                agent=self.current_agent,
+            )
+
+        except Exception:
+            pass  # Silently fail if right sidebar not available
+
+    async def _periodic_context_update(self) -> None:
+        """Periodically update context information while agent is busy."""
+        import asyncio
+
+        while self.agent_busy:
+            try:
+                # Update the right sidebar with current context
+                self._update_right_sidebar()
+
+                # Wait before next update (0.5 seconds for responsive updates)
+                await asyncio.sleep(0.5)
+            except asyncio.CancelledError:
+                # Task was cancelled, exit gracefully
+                break
+            except Exception:
+                # Silently handle any errors to avoid crashing the update loop
+                pass
+
+    def _start_context_updates(self) -> None:
+        """Start periodic context updates during agent execution."""
+        # Cancel any existing update worker
+        if self._context_update_worker is not None:
+            try:
+                self._context_update_worker.cancel()
+            except Exception:
+                pass
+
+        # Start a new background worker for context updates
+        self._context_update_worker = self.run_worker(
+            self._periodic_context_update(), exclusive=False
+        )
+
+    def _stop_context_updates(self) -> None:
+        """Stop periodic context updates."""
+        if self._context_update_worker is not None:
+            try:
+                self._context_update_worker.cancel()
+            except Exception:
+                pass
+            self._context_update_worker = None
+
+        # Do a final update when stopping
+        self._update_right_sidebar()
+
     def on_resize(self, event: Resize) -> None:
         """Handle terminal resize events to update responsive elements."""
         try:
@@ -1077,6 +1205,39 @@ class CodePuppyTUI(App):
                 # Log renderer stop errors but don't crash
                 self.add_system_message(f"Renderer stop error: {e}")
 
+    @on(ListView.Selected, "#history-list")
+    def on_history_list_selected(self, event: ListView.Selected) -> None:
+        """Handle clicks on history list items - show modal on double-click."""
+        import time
+
+        current_time = time.time()
+        current_index = event.list_view.index
+
+        # Check if this is a double-click (within 0.5 seconds and same item)
+        if (
+            self._last_history_click_time is not None
+            and self._last_history_click_index == current_index
+            and (current_time - self._last_history_click_time) < 0.5
+        ):
+            # This is a double-click - show the modal
+            try:
+                sidebar = self.query_one(Sidebar)
+                sidebar.current_history_index = current_index
+
+                from .components.command_history_modal import CommandHistoryModal
+
+                self.push_screen(CommandHistoryModal())
+            except Exception:
+                pass
+
+            # Reset tracking
+            self._last_history_click_time = None
+            self._last_history_click_index = None
+        else:
+            # This is a single click - just track it
+            self._last_history_click_time = current_time
+            self._last_history_click_index = current_index
+
     @on(HistoryEntrySelected)
     def on_history_entry_selected(self, event: HistoryEntrySelected) -> None:
         """Handle selection of a history entry from the sidebar."""
@@ -1118,11 +1279,14 @@ class CodePuppyTUI(App):
 async def run_textual_ui(initial_command: str = None):
     """Run the Textual UI interface."""
     # Always enable YOLO mode in TUI mode for a smoother experience
-    from code_puppy.config import set_config_value
+    from code_puppy.config import set_config_value, load_api_keys_to_environment
 
     # Initialize the command history file
     initialize_command_history_file()
 
+    # Load API keys from puppy.cfg into environment variables
+    load_api_keys_to_environment()
+
     set_config_value("yolo_mode", "true")
 
     app = CodePuppyTUI(initial_command=initial_command)
